@@ -1,6 +1,7 @@
 """
 Provides methods to work with the YNAB API.
 """
+import re
 from datetime import date
 from decimal import Decimal
 from typing import List, Optional, Dict
@@ -8,6 +9,8 @@ from mt940.models import Transaction as FinTSTransaction
 import requests
 
 YNAB_BASE_URL = "https://api.youneedabudget.com/v1"
+PAYPAL_PAYEE_REGEX = re.compile(r"^PayPal\s?\(Europe\)")
+PAYPAL_MEMO_REGEX = re.compile(r".*, Ihr Einkauf bei (.*)$")
 
 
 class YNABError(Exception):
@@ -53,6 +56,20 @@ class YNABClient:
             raise YNABError(body["error"]["detail"])
         return body["data"]["transaction_ids"]
 
+    def latest_transaction_date(self) -> Optional[date]:
+        """
+        Returns the date of the latest transaction.
+        """
+        headers = {"Authorization": "Bearer " + self.access_token}
+        path = "/budgets/" + self.budget_id + "/transactions"
+        response = requests.get(YNAB_BASE_URL + path, headers=headers)
+        body = response.json()
+        if response.status_code >= 400:
+            raise YNABError(body["error"]["detail"])
+        if not body["data"] or not body["data"]["transactions"]:
+            return None
+        return date.fromisoformat(body["data"]["transactions"][-1]["date"])
+
     def transform_transactions(
         self, transactions: List[FinTSTransaction],
     ) -> List[Dict[str, str]]:
@@ -82,6 +99,12 @@ class YNABClient:
                 f"YNAB:{milliunits_amount}:{transaction_date.isoformat()}:{occurence}"
             )
 
+            # If this is a PayPal transaction, try to get the Payee from the memo
+            if PAYPAL_PAYEE_REGEX.match(data["applicant_name"]):
+                payee = PAYPAL_MEMO_REGEX.match(data["purpose"])
+                if payee is not None:
+                    data["applicant_name"] = "PAYPAL " + payee.group(1)
+
             transformed.append(
                 {
                     "account_id": self.account_id,
